/******************************************************************************/
// Free implementation of Bullfrog's Dungeon Keeper strategy game.
/******************************************************************************/
/** @file engine_render.h
 *     Header file for engine_render.c.
 * @par Purpose:
 *     Rendering the 3D view functions.
 * @par Comment:
 *     Just a header file - #defines, typedefs, function prototypes etc.
 * @author   Tomasz Lis
 * @date     20 Mar 2009 - 30 Mar 2009
 * @par  Copying and copyrights:
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 */
/******************************************************************************/
#ifndef DK_ENGNREND_H
#define DK_ENGNREND_H

#include "bflib_basics.h"
#include "globals.h"
#include "bflib_render.h"

#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************/
#ifdef __cplusplus
#pragma pack(1)
#endif

#define BUCKETS_COUNT 704

struct MinMax;
struct Camera;
struct PlayerInfo;

struct BasicQ { // sizeof = 5
  struct BasicQ *next;
  unsigned char kind;
};

struct BasicUnk00 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short block;
    struct PolyPoint p1;
    struct PolyPoint p2;
    struct PolyPoint p3;
};

struct BasicUnk01 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short block;
    struct PolyPoint p1;
    struct PolyPoint p2;
    struct PolyPoint p3;
};

struct BasicUnk02 {
    struct BasicQ b;
    unsigned char colour;
    unsigned short x1;
    unsigned short y1;
    unsigned short x2;
    unsigned short y2;
    unsigned short x3;
    unsigned short y3;
};

struct BasicUnk03 {
    struct BasicQ b;
    unsigned char colour;
    unsigned short x1;
    unsigned short y1;
    unsigned short x2;
    unsigned short y2;
    unsigned short x3;
    unsigned short y3;
    unsigned char vf1;
    unsigned char vf2;
    unsigned char vf3;
    unsigned char field_15[3];
    unsigned char field_18;
    unsigned char field_19[3];
};

struct BasicUnk04 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short x1;
    unsigned short y1;
    unsigned short x2;
    unsigned short y2;
    unsigned short x3;
    unsigned short y3;
    unsigned short field_12;
    unsigned char uf1;
    unsigned char vf1;
    unsigned char uf2;
    unsigned char vf2;
    unsigned char uf3;
    unsigned char vf3;
};

struct BasicUnk05 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short x1;
    unsigned short y1;
    unsigned short x2;
    unsigned short y2;
    unsigned short x3;
    unsigned short y3;
    unsigned short field_12;
    unsigned char uf1;
    unsigned char vf1;
    unsigned char uf2;
    unsigned char vf2;
    unsigned char uf3;
    unsigned char vf3;
    unsigned char wf1;
    unsigned char wf2;
    unsigned char wf3;
};

struct BasicUnk06 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short x1;
    unsigned short y1;
    unsigned short x2;
    unsigned short y2;
    unsigned short x3;
    unsigned short y3;
    unsigned short field_12;
    unsigned char uf1;
    unsigned char vf1;
    unsigned char uf2;
    unsigned char vf2;
    unsigned char uf3;
    unsigned char vf3;
};

struct BasicUnk07 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short x1;
    unsigned short y1;
    unsigned short x2;
    unsigned short y2;
    unsigned short x3;
    unsigned short y3;
    unsigned short field_12;
    unsigned char uf1;
    unsigned char vf1;
    unsigned char uf2;
    unsigned char vf2;
    unsigned char uf3;
    unsigned char vf3;
    unsigned char wf1;
    unsigned char wf2;
    unsigned char wf3;
};

struct RotoSpr {
    struct BasicQ b;
    unsigned char field_5[3];
    long field_8;
    long field_C;
    long field_10;
    long field_14;
    char field_18;
    unsigned char field_19[3];
};

struct BasicUnk09 {
    struct BasicQ b;
    unsigned char subtype;
    unsigned short block;
    struct PolyPoint p1;
    struct PolyPoint p2;
    struct PolyPoint p3;
    long field_44;
    long field_48;
    long field_4C;
    long field_50;
    long field_54;
    long field_58;
    long field_5C;
    long field_60;
    long field_64;
};

struct BasicUnk10 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned char field_6;
    unsigned char field_7;
    struct PolyPoint p1;
    struct PolyPoint p2;
    struct PolyPoint p3;
};

struct JontySpr {  // BasicQ type 11,18
    struct BasicQ b;
    unsigned char field_5[3];
    long field_8;
    long field_C;
    long field_10;
    long field_14;
    unsigned char field_18;
    unsigned char field_19[3];
};

struct BasicUnk12 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short field_6;
    struct PolyPoint p1;
    struct PolyPoint p2;
    struct PolyPoint p3;
    struct PolyPoint p4;
    long field_58;
    unsigned short field_5C;
    unsigned char field_5E;
};

struct BasicUnk13 {
    struct BasicQ b;
    unsigned char field_5;
    unsigned short field_6;
    struct PolyPoint p;
    unsigned char field_19[3];
};

struct BasicUnk14 { // sizeof = 24
    struct BasicQ b;
    unsigned char field_5[3];
    struct Thing *thing;
    long field_C;
    long field_10;
    long field_14;
};

struct Number { // BasicQ type 16
    struct BasicQ b;
    unsigned char field_5[3];
    long x;
    long y;
    long lvl;
};

struct RoomFlag { // BasicQ type 17,19
    struct BasicQ b;
    unsigned char field_5;
    unsigned short lvl;
    long x;
    long y;
};

struct EngineCoord { // sizeof = 28
  long field_0;
  long field_4;
  unsigned short field_8;
  unsigned short field_A;
  long field_C;
  long x;
  long y;
  long z;
};

struct M33 { // sizeof = 48
  long r0[4];
  long r1[4];
  long r2[4];
};

struct EngineCol {
//TODO!!
};

struct MapVolumeBox { // sizeof = 24
  unsigned char field_0;
  unsigned char color;
  unsigned char field_2;
  long field_3;
  long field_7;
  long field_B;
  long field_F;
  long field_13;
  unsigned char field_17;
};

#ifdef __cplusplus
#pragma pack()
#endif

/******************************************************************************/
DLLIMPORT unsigned char *_DK_getpoly;
#define getpoly _DK_getpoly
DLLIMPORT unsigned char _DK_poly_pool[0x40000];
#define poly_pool _DK_poly_pool
DLLIMPORT unsigned char *_DK_poly_pool_end;
#define poly_pool_end _DK_poly_pool_end
DLLIMPORT struct BasicQ *_DK_buckets[BUCKETS_COUNT];
#define buckets _DK_buckets
DLLIMPORT Offset _DK_vert_offset[3];
#define vert_offset _DK_vert_offset
DLLIMPORT Offset _DK_hori_offset[3];
#define hori_offset _DK_hori_offset
DLLIMPORT Offset _DK_high_offset[3];
#define high_offset _DK_high_offset
DLLIMPORT long _DK_x_init_off;
#define x_init_off _DK_x_init_off
DLLIMPORT long _DK_y_init_off;
#define y_init_off _DK_y_init_off
DLLIMPORT long _DK_floor_pointed_at_x;
#define floor_pointed_at_x _DK_floor_pointed_at_x
DLLIMPORT long _DK_floor_pointed_at_y;
#define floor_pointed_at_y _DK_floor_pointed_at_y
DLLIMPORT long _DK_cells_away;
#define cells_away _DK_cells_away
DLLIMPORT long _DK_fade_max;
#define fade_max _DK_fade_max
DLLIMPORT long _DK_fade_scaler;
#define fade_scaler _DK_fade_scaler
DLLIMPORT long _DK_fade_way_out;
#define fade_way_out _DK_fade_way_out
DLLIMPORT struct MapVolumeBox _DK_map_volume_box;
#define map_volume_box _DK_map_volume_box
DLLIMPORT long _DK_map_angle;
#define map_angle _DK_map_angle
DLLIMPORT long _DK_map_roll;
#define map_roll _DK_map_roll
DLLIMPORT long _DK_map_tilt;
#define map_tilt _DK_map_tilt
DLLIMPORT long _DK_view_alt;
#define view_alt _DK_view_alt
DLLIMPORT long _DK_fade_min;
#define fade_min _DK_fade_min
DLLIMPORT long _DK_split_1;
#define split_1 _DK_split_1
DLLIMPORT long _DK_split_2;
#define split_2 _DK_split_2
DLLIMPORT long _DK_fade_range;
#define fade_range _DK_fade_range
DLLIMPORT long _DK_depth_init_off;
#define depth_init_off _DK_depth_init_off
DLLIMPORT int _DK_normal_shade_left;
#define normal_shade_left _DK_normal_shade_left
DLLIMPORT int _DK_normal_shade_right;
#define normal_shade_right _DK_normal_shade_right
DLLIMPORT long _DK_apos;
#define apos _DK_apos
DLLIMPORT long _DK_bpos;
#define bpos _DK_bpos
DLLIMPORT long _DK_ScrCenterX;
#define ScrCenterX _DK_ScrCenterX
DLLIMPORT long _DK_ScrWidth;
#define ScrWidth _DK_ScrWidth
DLLIMPORT long _DK_ScrHeight;
#define ScrHeight _DK_ScrHeight
DLLIMPORT long _DK_ScrCenterY;
#define ScrCenterY _DK_ScrCenterY
DLLIMPORT long _DK_split1at;
#define split1at _DK_split1at
DLLIMPORT long _DK_split2at;
#define split2at _DK_split2at
DLLIMPORT long _DK_max_i_can_see;
#define max_i_can_see _DK_max_i_can_see
DLLIMPORT long _DK_view_height_over_2;
#define view_height_over_2 _DK_view_height_over_2
DLLIMPORT long _DK_view_width_over_2;
#define view_width_over_2 _DK_view_width_over_2
DLLIMPORT int _DK_normal_shade_front;
#define normal_shade_front _DK_normal_shade_front
DLLIMPORT int _DK_normal_shade_back;
#define normal_shade_back _DK_normal_shade_back
//DLLIMPORT unsigned char _DK_temp_cluedo_mode;
//#define temp_cluedo_mode _DK_temp_cluedo_mode
DLLIMPORT long _DK_me_distance;
#define me_distance _DK_me_distance
DLLIMPORT short _DK_mx;
#define mx _DK_mx
DLLIMPORT short _DK_my;
#define my _DK_my
DLLIMPORT short _DK_mz;
#define mz _DK_mz
DLLIMPORT unsigned char _DK_engine_player_number;
#define engine_player_number _DK_engine_player_number
DLLIMPORT unsigned char _DK_player_bit;
#define player_bit _DK_player_bit
DLLIMPORT long _DK_gpoly_pro_enable_mode_ofs;
#define gpoly_pro_enable_mode_ofs _DK_gpoly_pro_enable_mode_ofs
DLLIMPORT long _DK_UseFastBlockDraw;
#define UseFastBlockDraw _DK_UseFastBlockDraw
DLLIMPORT unsigned char *_DK_gtblock_screen_addr;
#define gtblock_screen_addr _DK_gtblock_screen_addr
DLLIMPORT long _DK_gtblock_clip_width;
#define gtblock_clip_width _DK_gtblock_clip_width
DLLIMPORT long _DK_gtblock_clip_height;
#define gtblock_clip_height _DK_gtblock_clip_height
DLLIMPORT long _DK_gtblock_screen_width;
#define gtblock_screen_width _DK_gtblock_screen_width
/******************************************************************************/
extern unsigned char temp_cluedo_mode;
/******************************************************************************/
void do_a_plane_of_engine_columns_perspective(long a1, long a2, long a3, long a4);
void do_a_plane_of_engine_columns_cluedo(long a1, long a2, long a3, long a4);
void do_a_plane_of_engine_columns_isometric(long a1, long a2, long a3, long a4);
void find_gamut(void);
void fiddle_gamut(long a1, long a2);
void create_map_volume_box(long a1, long a2, long a3);
void rotpers_parallel_3(struct EngineCoord *epos, struct M33 *matx);
void rotate_base_axis(struct M33 *matx, short a2, unsigned char a3);
void fill_in_points_perspective(long a1, long a2, struct MinMax *mm);
void fill_in_points_cluedo(long a1, long a2, struct MinMax *mm);
void fill_in_points_isometric(long a1, long a2, struct MinMax *mm);
void frame_wibble_generate(void);
void setup_rotate_stuff(long a1, long a2, long a3, long a4, long a5, long a6, long a7, long a8);

void draw_engine_number(struct Number *num);
void draw_engine_room_flagpole(struct RoomFlag *rflg);

void update_engine_settings(struct PlayerInfo *player);
void display_drawlist(void);
void draw_view(struct Camera *cam, unsigned char a2);
void draw_frontview_engine(struct Camera *cam);
void gpoly_enable_pentium_pro(TbBool state);
/******************************************************************************/
#ifdef __cplusplus
}
#endif
#endif
